Package: munsell
Type: Package
Title: Utilities for Using Munsell Colours
Version: 0.5.1
Author: Charlotte Wickham <cwickham@gmail.com>
Maintainer: Charlotte Wickham <cwickham@gmail.com>
Description: Provides easy access to, and manipulation of, the Munsell 
    colours. Provides a mapping between Munsell's 
    original notation (e.g. "5R 5/10") and hexadecimal strings suitable 
    for use directly in R graphics. Also provides utilities 
    to explore slices through the Munsell colour tree, to transform 
    Munsell colours and display colour palettes.
Suggests: ggplot2, testthat
Imports: colorspace, methods
License: MIT + file LICENSE
URL: https://cran.r-project.org/package=munsell,
        https://github.com/cwickham/munsell/
RoxygenNote: 7.3.1
Encoding: UTF-8
BugReports: https://github.com/cwickham/munsell/issues
NeedsCompilation: no
Packaged: 2024-04-01 20:42:09 UTC; charlottewickham
Repository: CRAN
Date/Publication: 2024-04-01 23:40:10 UTC
Built: R 4.4.0; ; 2026-07-18 11:09:41 UTC; unix

Package: munsell
Type: Package
Title: Utilities for Using Munsell Colours
Version: 0.5.0
Author: Charlotte Wickham <cwickham@gmail.com>
Maintainer: Charlotte Wickham <cwickham@gmail.com>
Description: Provides easy access to, and manipulation of, the Munsell 
    colours. Provides a mapping between Munsell's 
    original notation (e.g. "5R 5/10") and hexadecimal strings suitable 
    for use directly in R graphics. Also provides utilities 
    to explore slices through the Munsell colour tree, to transform 
    Munsell colours and display colour palettes.
Suggests: ggplot2, testthat
Imports: colorspace, methods
License: MIT + file LICENSE
URL: https://cran.r-project.org/package=munsell,
        https://github.com/cwickham/munsell/
RoxygenNote: 6.0.1
NeedsCompilation: no
Packaged: 2018-06-11 23:15:15 UTC; wickhamc
Repository: CRAN
Date/Publication: 2018-06-12 04:29:06 UTC

# Generated by roxygen2: do not edit by hand

export(check_mnsl)
export(chroma_slice)
export(complement)
export(complement_slice)
export(darker)
export(desaturate)
export(fix_mnsl)
export(hue_slice)
export(hvc2mnsl)
export(in_gamut)
export(lighter)
export(mnsl)
export(mnsl2hex)
export(mnsl2hvc)
export(mnsl_hues)
export(pbgyr)
export(plot_closest)
export(plot_hex)
export(plot_mnsl)
export(rgb2mnsl)
export(rygbp)
export(saturate)
export(seq_mnsl)
export(text_colour)
export(value_slice)
import(colorspace)
importFrom(methods,as)
importFrom(stats,na.exclude)

<!-- README.md is generated from README.Rmd. Please edit that file -->

<figure>
<img src="http://cranlogs.r-pkg.org/badges/last-week/munsell"
alt="Downloads" />
<figcaption aria-hidden="true">Downloads</figcaption>
</figure>

<!-- badges: start -->

[![R-CMD-check](https://github.com/cwickham/munsell/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/cwickham/munsell/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

# munsell

The `munsell` package provides easy access to, and manipulation of, the
Munsell colours. The `munsell` package provides a mapping between
Munsell’s original notation (e.g. “5R 5/10”) and hexadecimal sRGB
strings suitable for use directly in R graphics. The package also
provides utilities to explore slices through the Munsell colour tree, to
transform Munsell colours and display colour palettes.

Munsell devised his system of colour notation to match the three
perceptual dimensions of colour: hue, value and chroma. His notation
provides a naming scheme to colours that eases the choice of color
according to a specific purpose. His century old advice is still
relevant for the producers of statistical graphics and the munsell
package aims to enable user to easily follow it.

`munsell` relies directly on the published tables in Newhall, Nickerson,
and Judd (1943) of CIE XYZ (Illuminant C) values for Munsell colours.
These tables were the result of colour matching studies on Munsell’s
color samples along with some smoothing and extrapolation with Munsell’s
goal of perceptually uniform spacing in mind.

Currently the `munsell` package only includes hue in steps of 2.5, value
in steps of 2 and chroma in steps of 1, corresponding directly to the
entries in Table 1 in Newhall, Nickerson, and Judd (1943). If you
require a more continuous mapping, see
[munsellinter](https://r-forge.r-project.org/projects/munsellinterpol/).

# Usage

Functions in `munsell` fall into three basic use categories: specifying
Munsell colours, altering Munsell colours and exploring the Munsell
color space.

The code below relies on the development version of `munsell`, get it
with:

``` r
devtools::install_github("cwickham/munsell")
```

## Color specification

Following Munsell, specifying colours is done with a specific string
format: “H V/C” where H is a hue code (see `mnsl_hues()` for a list of
those available, excluding “N”), V an integer in $[0, 10]$ specifying
value, and C an even integer specifying chroma. The `mnsl` function
takes the string and returns a hexadecimal RGB representation:

``` r
library(munsell)
mnsl("5R 5/10")
#> [1] "#C65858"
```

Visually examining a colour can either be done by using `mnsl` with a
base plotting call, or using `plot_mnsl` which plots colour swatches
using `ggplot2`:

``` r
plot.new()
rect(0, 0, 1 ,1 , col = mnsl("5R 5/10"))
plot_mnsl("5R 5/10")
```

## Colour manipulation

`munsell` provides convenience functions that alter a colour by taking
steps in the hue, value and chroma dimensions: `rygbp`, `pbgyr`,
`lighter`, `darker`, `saturate` and `desaturate`.

``` r
my_blue <- "5PB 5/8"
p <- plot_mnsl(c(
  lighter(my_blue, 2),      my_blue,   darker(my_blue, 2),
  desaturate(my_blue, 2),   my_blue,   saturate(my_blue, 2),
  rygbp(my_blue, 2),        my_blue,   pbgyr(my_blue, 2)))
p
```

![](man/figures/README-manipulate-blue-1.png)<!-- -->

Each function optionally takes the number of steps to take in the
dimension and consequently are easily used to create scales in a
particular dimension.

``` r
p <- plot_mnsl(sapply(0:6, darker, col = "5PB 7/4"))
p + ggplot2::facet_wrap(~ num, nrow = 1)
```

![](man/figures/README-palette-1.png)<!-- -->

## Colour space exploration

Slices through the colour space of constant hue, chroma or value can be
displayed using the functions: `hue_slice`, `chroma_slice` and
`value_slice`. Additionally `complement_slice` displays a slice of
constant hue, alongside a slice of its complement, the hue that is on
the opposite side of the colour sphere to that specified.

``` r
complement_slice("5R")
```

![](man/figures/README-complement-slice-1.png)<!-- -->

# References

<div id="refs" class="references csl-bib-body hanging-indent"
entry-spacing="0">

<div id="ref-Newhall:1943aa" class="csl-entry">

Newhall, Sidney M, Dorothy Nickerson, and Deane B Judd. 1943. “Final
Report of the OSA Subcommittee on the Spacing of the Munsell Colors.”
*JOSA* 33 (7): 385–411.

</div>

</div>

### explore the mapping 
getmunsellmap <- function(){
  require(colorspace)
  col.map <- read.table("real.dat",  header = TRUE)

  # correct sequence
  # 1. convert xyY to XYZ
  # 2. convert to XYZ to use correct reference white (C to D65)
  # 3. convert XYZ (D65) to sRGB

  # 1. convert to XYZ
  # http://www.brucelindbloom.com/Eqn_xyY_to_XYZ.html
  # Y needs to be scaled down by 100
  col.map <- within(col.map, {
  	Y <- Y/100
  	X <- x * Y / y
  	Z <- ((1 - x - y) * Y) / y
  })

  # 2. convert to XYZ to use correct reference white (C to D65)
  # http://www.brucelindbloom.com/Eqn_ChromAdapt.html
  # using Bradford method
  Bradford.C.D65 <- matrix(c(0.990448, -0.012371, -0.003564, -0.007168, 1.015594, 0.006770, -0.011615, -0.002928, 0.918157), ncol=3, byrow=TRUE)
  col.map[ , c("X", "Y", "Z")] <- as.matrix(col.map[, c("X", "Y", "Z")]) %*%
    Bradford.C.D65
  
  # 3. Use colorspace methods to convert XYZ to hex (sRGB)  
  col.map$hex <- hex(XYZ(100 * as.matrix(col.map[, c("X", "Y", "Z")])))

  cols <- c("R", "YR", "Y", "GY", "G", "BG", "B", "PB", "P", "RP")
  ints <- seq(2.5, 10, 2.5)
  col.map$h <- factor(col.map$h,  levels = paste(rep(ints, 10), 
    rep(cols, each = 4), sep = ""))

  # from here: http://wiki.laptop.org/go/Munsell
  grey.map <- read.table("greys.dat",  header = TRUE)
  grey.map$hex <-  hex(sRGB(as.matrix(1/255 * grey.map[, c("r", "b", "g")])))

  munsell.map <- rbind(grey.map[, c("h", "C", "V", "hex")], 
    col.map[, c("h", "C", "V", "hex")])
  names(munsell.map) <- c("hue", "chroma", "value", "hex")
  munsell.map$name <- paste(munsell.map$hue, " ", munsell.map$value, "/", munsell.map$chroma,  sep = "") 
  munsell.map$name[is.na(munsell.map$hex)] <- NA

  not.miss <- subset(munsell.map, !is.na(hex))
  not.miss <- cbind(not.miss, as(hex2RGB(not.miss$hex), "LUV")@coords)
  munsell.map <- merge(munsell.map, not.miss,  all.x = TRUE)
  munsell.map[munsell.map$name == "N 0/0" & !is.na(munsell.map$name), 
    c("L", "U", "V")] <- c(0, 0, 0)
  
  more.greys <- expand.grid(hue = unique(col.map$h), chroma = 0, value = 0:10)
  munsell.map <- rbind(munsell.map, 
    merge(more.greys, 
      munsell.map[munsell.map$hue == "N", c("chroma", "value", "hex", "name", "L", "U", "V")]))
  
  save(munsell.map,  file  = "../../R/sysdata.rda")
}

#' Munsell colour system.
#' 
#' @description 
#' This package makes it easy to access and manipulate the colours in the 
#' munsell colour system. The conversion from munsell specifications to sRGB based on the renotation data from  \url{http://www.cis.rit.edu/mcsl/online/munsell.php} which is a digitization of Table 1 in Newhall, Nickerson & Judd (1943).  The code for conversion can be found in the package directory in inst/raw/getmunsellmap.r
#' @references S. M. Newhall, D. Nickerson, and D. B. Judd. Final report of the O.S.A. subcommittee on the spacing of the munsell colors. J. Opt. Soc. Am., 33(7):385-411, 07 1943.
#' @references Munsell Renotation Data, RIT Munsell Color Science Laboratory.  \url{http://www.cis.rit.edu/mcsl/online/munsell.php}
#' @name munsell
#' @aliases munsell package-munsell
#' @import colorspace
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

#' Checks for valid Munsell colours
#'
#' Checks user supplied munsell specification for validity.  
#' I.e. colour is of form "h v/c" and h,  v and c take valid values.
#' @param col a character vector representing Munsell colours.
#' @return  a character vector containing the input colours.  If any colours
#' were outside the gamut they will be represented by NA.
#' @export
#' @examples 
#' check_mnsl(c("5R 5/8","2.5R 9/28"))
#' @keywords internal
#' @importFrom methods as
#' @importFrom stats na.exclude
check_mnsl <- function(col){
  col_na <- na.exclude(col)
  col <- toupper(as.vector(col_na))
  # check format
  right.format <- grep("^[N]|([0-9]?.?[0-9][A-Z]{1,2})[ ][0-9]?.?[0-9]/[0-9]?.?[0-9]{1,2}$",
    col)
  if (length(right.format) != length((col))) {
    if (length(right.format) == 0) {bad.cols <- paste(col,  collapse = ", ")}
    else {bad.cols <- paste(col[-right.format],  collapse = ", ")}
    stop("some of your colours are not correctly formatted:",  bad.cols)
  }
  #check hues
  hues <- gsub("[0-9 /.]", "", col)
  act.hues <- c("N", "R", "YR", "Y", "GY", "G", "BG", "B", "PB", "P", "RP")
  good.hue <-  hues %in% act.hues
  if (!all(good.hue)){
    bad.hue <- paste(hues[!good.hue], "in", col[!good.hue],  
      collapse = "; ")
    act.hue.str <- paste(act.hues, collapse = ", ")
    stop("you have specified invalid hue names: ", bad.hue, 
      "\n hues should be one of ", act.hue.str)
  }
  col.split <- lapply(strsplit(col, "/"), 
     function(x) unlist(strsplit(x, " ")))
  col.split <- lapply(col.split, gsub, pattern = "[A-Z]", replacement = "")
  step <- as.numeric(sapply(col.split, "[", 1))
  values <- as.numeric(sapply(col.split, "[", 2))
  chromas <- as.numeric(sapply(col.split, "[", 3))
  
  act.steps <- c(seq(2.5, 10, by = 2.5),  NA)
  good.step <- step %in% act.steps
  if(!all(good.step)){
    bad.step <- paste(step[!good.step], "in", col[!good.step],  
      collapse = "; ")
    act.step.str <- paste(act.steps,  collapse = ", ")
    stop("you have specified invalid hue steps: ", bad.step, 
       "\n hues steps should be one of ", act.step.str)
  }
  good.value <- values == round(values) & values <= 10 & values >= 0
  if(!all(good.value)) {
    bad.value <- paste(values[!good.value], "in", col[!good.value],  
      collapse = "; ")
    stop("some colours have values that are not integers between 0 and 10: ", bad.value)
  }
  good.chroma <- (chromas %% 2) == 0 
  if(!all(good.chroma)) {
    bad.chroma <- paste(chromas[!good.chroma], "in", col[!good.chroma],  
      collapse = "; ")
    stop("some colours have chromas that are not multiples of two: ",
      bad.chroma)
  }
  na_handle(col_na, col)
}

#' Checks if a Munsell colour is defined in RGB space
#'
#' Not all possible correctly formatted Munsell colours result in a colour
#' representable in RGB space.  This function checks if the colour is
#' representable.  
#' @param col a character vector representing Munsell colours.
#' @param fix passed on to \code{\link{fix_mnsl}}. Use \code{fix = TRUE} to
#' fix "bad" colours
#' @return  a character vector containing the input colours.  If any colours
#' were outside the gamut they will be represented by NA.
#' @export
#' @importFrom stats na.exclude
#' @examples 
#' in_gamut(c("5R 5/8","2.5R 9/28"))
#' @keywords internal
in_gamut <- function(col, fix = FALSE){
  col <- na.exclude(col)
  positions <- match(col, munsell.map$name)
  hex <- munsell.map[positions, "hex"]
  if(any(is.na(hex))){
    bad.colours <- paste(col[is.na(hex)], collapse = ", ")
    if(!fix){
      warning("some specified colours are undefined. You could try fix = TRUE")
      col[is.na(hex)] <- NA
      }
    else{
      col[is.na(hex)] <- fix_mnsl(col[is.na(hex)])
    }
  }
  na_handle(col, as.vector(col))
}
#' Fix an undefined Munsell colour
#'
#' Takes correctly specified but undefined colours and outputs something
#' sensible.  Normally this happens when the chroma is too high.  So,  here
#' sensible means the colour with the same hue and value and maximum defined
#' chroma.
#' @param col a character vector representing Munsell colours.
#' @return  a character vector containing the fixed colours.
#' @export
#' @examples 
#' fix_mnsl(c("5R 5/8","2.5R 9/28"))
#' @keywords internal
fix_mnsl <- function(col){
  col.split <- lapply(strsplit(col, "/"), 
     function(x) unlist(strsplit(x, " ")))
  max.chroma <- function(colour.args){
    hue.value <- munsell.map[munsell.map$hue == colour.args[1] & 
        munsell.map$value == colour.args[2] & !is.na(munsell.map$hex),] 
    hue.value[which.max(hue.value$chroma), "name"]
  }
  unlist(lapply(col.split, max.chroma))
}

#' Munsell hues
#'
#' Returns a character vector of the Munsell hues in hue order starting at 2.5R and excluding grey ("N").
#' @return  a character vector containing the hue values.
#' @export
#' @examples 
#' mnsl_hues()
mnsl_hues <- function(){
 as.character(unique(munsell.map$hue)[-1])
}

#' Make a munsell colour lighter
#'
#' Increases the value of the Munsell colour. 
#' @param col character vector of Munsell colours
#' @param steps number of steps to take in increasing value
#' @return character vector of Munsell colours
#' @export
#' @importFrom stats na.exclude
#' @examples 
#' lighter("5PB 2/4")
#' cols <- c("5PB 2/4", "5Y 6/8")
#' p <- plot_mnsl(c(cols, lighter(cols), lighter(cols, 2)))
#' p + ggplot2::facet_wrap(~ names, ncol = 2)
#' # lighter and darker are usually reversible
#' lighter(darker("5PB 2/4"))
#' # unless you try to pass though white or black
#' lighter(darker("5PB 1/4"))
lighter <- function(col, steps = 1){
  col <- na.exclude(col)
  col_hvc <- mnsl2hvc(as.vector(col))
  
  col_hvc[, "value"] <- col_hvc[, "value"] + steps
  # check edge cases
  whites <- col_hvc[, "value"] >= 10
  blacks <- col_hvc[, "value"] <= 0
  if (any(whites | blacks)){
    col_hvc[whites, "hue"] <- "N"
    col_hvc[whites, "value"] <- 10
    col_hvc[whites, "chroma"] <- 0
    col_hvc[blacks, "hue"] <- "N"
    col_hvc[blacks, "value"] <- 0
    col_hvc[blacks, "chroma"] <- 0
  }
  na_handle(col, hvc2mnsl(col_hvc))
}

na_handle <- function(naobj, res){
  nas <- attr(naobj, "na.action")
  if(is.null(nas)) return(res)
  
  if (is.vector(res) & is.vector(as.vector(naobj))){
    keep <- rep(NA, length(naobj) + length(nas))
    stopifnot(length(naobj) == length(res))
    keep[-nas] <- 1:length(res)
    result <- res[keep] 
  } else if(is.data.frame(res) & is.vector(as.vector(naobj))){
    keep <- rep(NA, length(naobj) + length(nas))
    stopifnot(length(naobj) == nrow(res))
    keep[-nas] <- 1:nrow(res)
    result <- res[keep, ]
  } else if(is.vector(res) & is.data.frame(naobj)){
    keep <- rep(NA, nrow(naobj) + length(nas))
    stopifnot(nrow(naobj) == length(res))
    keep[-nas] <- 1:length(res)
    result <- res[keep]
  } else if (is.data.frame(res) & is.data.frame(naobj)){
    keep <- rep(NA, nrow(naobj) + length(nas))
    stopifnot(nrow(naobj) == nrow(res))
    keep[-nas] <- 1:nrow(res)
    result <- res[keep, ]
  }
  
  result
}

#' Make a munsell colour darker
#'
#' Decreases the value of the Munsell colour by 1.
#' @param col character vector of Munsell colours
#' @param steps number of steps to take in decreasing value
#' @return character vector of Munsell colours
#' @export
#' @examples 
#' darker("5PB 3/4")
#' cols <- c("5PB 3/4", "5Y 7/8")
#' p <- plot_mnsl(c(cols, darker(cols), darker(cols, 2)))
#' p + ggplot2::facet_wrap(~ names, ncol = 2)
darker <- function(col, steps = 1){
  lighter(col, steps = -steps)
}

#' Make a munsell colour more saturated
#'
#' Increases the chroma of the Munsell colour by step steps (multiples of 2).
#' @param col character vector of Munsell colours
#' @param steps number of steps to take in increasing chroma
#' @return character vector of Munsell colours
#' @export
#' @importFrom stats na.exclude
#' @examples
#' saturate("5PB 2/4")
#' cols <- c("5PB 2/2", "5Y 7/6")
#' p <- plot_mnsl(c(cols, saturate(cols), saturate(cols, 2)))
#' p + ggplot2::facet_wrap(~ names, ncol = 2)
saturate <- function(col, steps = 1){
  col <- na.exclude(col)
  
  col_hvc <- mnsl2hvc(as.vector(col))
  col_hvc[, "chroma"] <- col_hvc[, "chroma"] + 2*steps
  greys <- col_hvc[, "chroma"] <= 0
  if (any(greys)){
    col_hvc[greys, "hue"] <- "N"
    col_hvc[greys, "chroma"] <- 0
  }  
  na_handle(col, hvc2mnsl(col_hvc))
}

#' Make a munsell colour less saturated
#'
#' Decreases the chroma of the Munsell colour by one step steps (multiples of 2).
#' @param col character vector of Munsell colours
#' @param steps number of steps to take in decreasing chroma
#' @return character vector of Munsell colours
#' @export
#' @examples 
#' desaturate("5PB 2/4")
#' cols <- c("5PB 2/6", "5Y 7/8")
#' p <- plot_mnsl(c(cols, desaturate(cols), desaturate(cols, 2)))
#' p + ggplot2::facet_wrap(~ names, ncol = 2)
desaturate <- function(col, steps = 1){
  saturate(col, steps = -steps)
}

#' Find the complement of a munsell colour 
#'
#' Finds the munsell colour with the same chroma and value but on the opposite
#' side of the hue circle. The complement is not defined 
#' for greys (hue == "N"), and the function returns the grey untransformed.
#' @param col character vector of Munsell colours
#' @param ... deprecated
#' @return character vector of Munsell colours
#' @export
#' @importFrom stats na.exclude
#' @examples 
#' complement("5PB 2/4")
#' cols <- c("5PB 2/4", "5Y 7/8")
#' plot_mnsl(c(cols, complement(cols)))
complement <- function(col, ...){
  
  if(!missing(...)) warning("Passing `...` to `complement()` is deprecated",
    call. = FALSE)
  
  col <- na.exclude(col)
  
  col_hvc <- mnsl2hvc(as.vector(col))
  greys <- col_hvc[, "hue"] == "N"
  inds <- match(col_hvc$hue, mnsl_hues())
  col_hvc[, "hue"] <-  mnsl_hues()[((inds + 20 -1) %% 40) + 1]
  if (any(greys)){
    warning("Complement not defined for greys")
    col_hvc[greys, "hue"] <- "N"
  }
  na_handle(col, hvc2mnsl(col_hvc))
}

#' Generate a sequence of Munsell colours
#'
#' Generates a sequence of Munsell colours.  The sequence is generated by 
#' finding the closest munsell colours to a equidistant sequence of colours in  #' LUV space.
#' @param from character string of first Munsell colour
#' @param to character string of last Munsell colour
#' @param n number of colours in sequence
#' @param fix Should colours outside of the gamut be fixed? 
#' Passed on to \code{\link{fix_mnsl}}
#' @return character vector of Munsell colours
#' @export
#' @importFrom methods as
#' @examples
#' seq_mnsl("5R 2/4", "5R 5/16", 4)
#' plot_mnsl(seq_mnsl("5R 2/4", "5R 5/16", 4))
#' plot_mnsl(seq_mnsl("5R 5/6", 
#'   complement("5R 5/6"), 5))
seq_mnsl <- function(from, to, n, fix = FALSE){
  cols <- in_gamut(c(from, to), fix = fix)
  if(any(is.na(cols))) stop("Colors must be in gamut")
  in.LUV <- munsell.map[match(cols, munsell.map$name), c("L", "U", "V")]
  LUV.seq <- matrix(c(seq(in.LUV$L[1], in.LUV$L[2],  length = n), 
    seq(in.LUV$U[1], in.LUV$U[2],  length = n), 
    seq(in.LUV$V[1], in.LUV$V[2],  length = n)),  ncol = 3)
  rgb2mnsl(as(LUV(LUV.seq), "sRGB")@coords)
}



#' Change the hue of a munsell colour
#'
#' Moves the hue of a munsell colour in the direction red->yellow->green->blue->purple->red
#' @param col character vector of Munsell colours
#' @param steps number of hue steps to take
#' @return character vector of Munsell colours
#' @export
#' @importFrom stats na.exclude
#' @examples 
#' my_red <- "10R 4/8"
#' rygbp(my_red)
#' plot_mnsl(c(my_red, rygbp(my_red, 2), rygbp(my_red, 4)))
rygbp <- function(col, steps = 1){
  col <- na.exclude(col)
  
  col_hvc <- mnsl2hvc(as.vector(col))
  greys <- col_hvc[, "hue"] == "N"

  inds <- match(col_hvc$hue, mnsl_hues())
  col_hvc[, "hue"] <-  mnsl_hues()[((inds + steps -1) %% 40) + 1]
  if (any(greys)){
    warning("Greys returned untransformed")
    col_hvc[greys, "hue"] <- "N"
  }
  
  na_handle(col, hvc2mnsl(col_hvc))
}

#' Change the hue of a munsell colour
#'
#' Moves the hue of a munsell colour in the direction purple->blue->green->yellow->red->purple
#' @param col character vector of Munsell colours
#' @param steps number of hue steps to take
#' @return character vector of Munsell colours
#' @export
#' @examples 
#' my_red <- "2.5R 4/8"
#' pbgyr(my_red)
#' plot_mnsl(c(my_red, pbgyr(my_red, 2), pbgyr(my_red, 4)))
pbgyr <- function(col, steps = 1){
  rygbp(col, steps = -steps)
}

context("Lightening/darkening colours")

test_that("Lightening a light colour gives white", {
  expect_equal(lighter("5PB 9/4"), "N 10/0")
  expect_equal(lighter("N 9/0"), "N 10/0")
  expect_equal(lighter("N 10/0 "), "N 10/0")
  expect_equal(lighter(c("N 9/0 ", "N 10/0 ")), c("N 10/0", "N 10/0"))
})
  
test_that("Darkening a dark colour gives black", {
  expect_equal(darker("N 0/0"), "N 0/0")
  expect_equal(darker("5PB 1/4"), "N 0/0")
})

test_that("Negative darkening lightens", {
  expect_equal(lighter("5PB 2/4", -1), darker("5PB 2/4", 1))
})

context("Saturate/desaturate colours")

test_that("Saturation edge cases", {
  expect_equal(desaturate("5PB 2/2"), "N 2/0")
  expect_equal(saturate("5PB 2/32"), "5PB 2/34")
})

test_that("Saturation/desaturate opposites", {
  expect_equal(desaturate("5PB 2/4"), saturate("5PB 2/4", -1))
  expect_equal(desaturate(saturate("5PB 2/10")), "5PB 2/10")
})

context("Complement colours")

test_that("Complement",{
  expect_equal(complement("2.5R 2/2"), "2.5BG 2/2")
  expect_equal(complement("10G 2/2"), "10RP 2/2")
  expect_warning(complement("N 10/0"), "grey")
})

context("Hues")

test_that("hue edges",{
  expect_equal(pbgyr("2.5R 2/2"), "10RP 2/2")
  expect_equal(rygbp("10RP 2/2"), "2.5R 2/2")
  expect_warning(rygbp("N 10/0"), "[Gg]rey")
})

context("Handling NAs")
test_that(" NA handler", {
expect_equivalent(na_handle(na.exclude(NA), numeric(0)), as.numeric(NA))
# vector vector
expect_equal(na_handle(na.exclude(c(NA, 1:2)), 3:4), c(NA, 3, 4))
# vector dataframe
expect_equivalent(na_handle(na.exclude(c(NA, 1:2)), data.frame(x = 1:2, y = 4:5)),
  data.frame(x = c(NA, 1:2), y = c(NA, 4:5)))
# dataframe vector
expect_equivalent(na_handle(na.exclude(data.frame(x = c(NA, 1:2), y = c(NA, 4:5))), 1:2),
  c(NA, 1:2))
# dataframe dataframe 
expect_equivalent(na_handle(na.exclude(data.frame(x = c(NA, 1:2), y = c(NA, 4:5))), 
    data.frame(x = 1:2, y = 4:5)),
  data.frame(x = c(NA, 1:2), y = c(NA, 4:5)))
})

test_that("single NA", {
  expect_error(lighter(NA), "zero")
  expect_error(saturate(NA),"zero")
  expect_error(rygbp(NA), "zero")
  expect_error(complement(NA), "zero")
})

test_that("NA with colour", {
  expect_equal(lighter(c(NA, "10RP 2/2")), c(NA, lighter("10RP 2/2")))
  expect_equal(saturate(c(NA, "10RP 2/2")), c(NA, saturate("10RP 2/2")))
  expect_equal(rygbp(c(NA, "10RP 2/2")), c(NA, rygbp("10RP 2/2")))
  expect_equal(complement(c(NA, "10RP 2/2")), c(NA, complement("10RP 2/2")))
})

context("Testing missing value conversion")

test_that("NAs handled in convert", {  
  expect_error(mnsl2hvc(c(NA)), "zero")
  expect_equal(hvc2mnsl(mnsl2hvc(c(NA, "10RP 2/2"))), c(NA, "10RP 2/2"))
})

test_that("NAs handled in checks", {
  expect_equal(check_mnsl(NA), as.character(NA))
  expect_equal(in_gamut(NA), as.logical(NA)) #wtf
  expect_equal(check_mnsl(c(NA, "10RP 2/2")), c(NA, "10RP 2/2"))
})

context("Out of gamut colors")

test_that("Fix gets passed along to `in_gamut()`", {
  expect_equal(mnsl("5PB 5/10", fix = TRUE), mnsl("5PB 5/10"))
  expect_equal(mnsl("5PB 5/14", fix = TRUE), mnsl("5PB 5/12"))
})

test_that("Converting out of gamut colors generate warnings", {
  expect_warning(mnsl("5PB 5/14"), "fix")
  expect_warning(mnsl(rygbp("2.5G 8/12")), "fix")
  
})

test_that("Altering out of gamut colors don't generate warnings", {
  expect_equal(lighter("5R 6/12"), "5R 7/12")
  expect_equal(rygbp("2.5G 8/12"), "5G 8/12")
  expect_equal(complement("2.5RP 8/12"), "2.5G 8/12")
})

#' Converts a Munsell colour to hex
#'
#' Take a character string representation of a Munsell colour and returns the 
#' hex specification of that colour
#'
#' Munsell colours are specified by hue, value and chroma.  They 
#' take a form like "5PB 5/10" where the first characters represent the
#' hue, followed by a space then the value and chroma separated by a "/". In
#' this package value should be an integer in 0:10 and chroma an even number
#' at most 24.  Note that not all possible specifications result in 
#' representable colours.  
#' @param col a character string representing a Munsell colour.
#' @param ... passed on to \code{\link{in_gamut}}. Use \code{fix = TRUE} to
#' fix "bad" colours
#' @return a character string specification of a hex colour
#' @seealso \code{\link{check_mnsl}},\code{\link{in_gamut}}, \code{\link{hvc2mnsl}}
#' @aliases mnsl2hex mnsl
#' @export mnsl2hex mnsl
#' @examples
#' mnsl2hex("5PB 5/10")
#' # use a munsell colour in a plot
#' plot.new()
#' rect(0, 0, 1 ,1 , col = mnsl("5R 5/10"))
mnsl <- function(col, ...){
  col <- check_mnsl(col)
  col <- in_gamut(col, ...)
  positions <- match(col, munsell.map$name)
  munsell.map[positions, "hex"]
}
mnsl2hex <- mnsl

#' Converts a hue, chroma and value to a Munsell colour
#'
#' Takes separate specifications of hue, value and chroma and returns the 
#' text specification of that colour.
#'
#' Munsell colours are specified by hue, value and chroma.  They 
#' take a form like "5PB 5/10" where the first characters represent the
#' hue, followed by a space then the value and chroma separated by a "/". In
#' this package value should be an integer in 0:10 and chroma an even number
#' at most 24.  Note that not all possible specifications result in 
#' representable colours.  Regular recycling rules apply.
#' @param hue a character vector of Munsell hues, or a 3 column data frame
#' containing the hue value and chroma levels
#' @param value a numeric vector of values
#' @param chroma a numeric vector of chromas
#' @param ... passed on to \code{\link{check_mnsl}}. Use \code{fix = TRUE} to
#' fix "bad" colours
#' @return a character string specification of a hex colour
#' @seealso \code{\link{check_mnsl}}, \code{\link{mnsl2hex}}
#' @export
#' @importFrom stats na.exclude
#' @examples
#' hvc2mnsl("5PB", 5, 10)
#' # All values of 5PB with chroma 10
#' hvc2mnsl("5PB", 1:9, 10) # note some are undefined
#' plot_mnsl(hvc2mnsl("5PB", 1:9, 10))
hvc2mnsl <- function(hue, value = NULL, chroma = NULL, ...){
  if(!(is.null(value) == is.null(chroma))) stop("specify both value and chroma")
  hcv <- hue
  if(!is.null(value)) {
    hcv <- cbind(hcv, value, chroma)
  }
  hcv <- na.exclude(hcv)
  selected <- paste(hcv[, 1], " ", hcv[, 2], "/", hcv[, 3],  sep = "")
  selected <- check_mnsl(selected, ...)
  na_handle(hcv, selected)
}

#' Converts a Munsell colour to a hue, chroma and value triplet
#'
#' Takes a text specification of a Munsell colour and returns
#' the hue, chroma and value triplet.
#'
#' Munsell colours are specified by hue, value and chroma.  They 
#' take a form like "5PB 5/10" where the first characters represent the
#' hue, followed by a space then the value and chroma separated by a "/". In
#' this package value should be an integer in 0:10 and chroma an even number
#' at most 24.  Note that not all possible specifications result in 
#' representable colours. 
#' @param col a character vector of Munsell colours
#' @param ... passed on to \code{\link{check_mnsl}}. Use \code{fix = TRUE} to
#' fix "bad" colours
#' @return a data frame with named columns hue, value and chroma containing the hue,
#' value and chroma levels.
#' @seealso \code{\link{check_mnsl}}, \code{\link{hvc2mnsl}}
#' @importFrom stats na.exclude
#' @export
#' @examples
#' mnsl2hvc("5PB 5/10")
#' hvc2mnsl(mnsl2hvc("5PB 5/10"))
mnsl2hvc <- function(col, ...){
  col <- check_mnsl(col, ...)
  col <- na.exclude(col)
  if (length(col) == 0) stop("zero non-missing colours")
  col.split <- lapply(strsplit(col, "/"), 
    function(x) unlist(strsplit(x, " ")))
  col_mat <- data.frame(do.call(rbind, col.split),
    stringsAsFactors = FALSE)
  colnames(col_mat) <- c("hue", "value", "chroma")
  col_mat[, "value"] <- as.numeric(col_mat[, "value"])
  col_mat[, "chroma"] <- as.numeric(col_mat[, "chroma"])
  na_handle(col, col_mat)
}
#' Converts an sRGB colour to Munsell
#'
#' Finds the closest Munsell colour (in LUV space) to the specified sRGB colour
#'
#' @param R a numeric vector of red values or a 3 column matrix with the
#' proportions R,  G,  B in the columns.
#' @param G numeric vector of green values
#' @param B numeric vector of blue values
#' @seealso \code{\link{plot_closest}}
#' @export
#' @importFrom methods as
#' @examples
#' rgb2mnsl(0.1, 0.1, 0.3)
#' rgb2mnsl(matrix(c(.1, .2, .4, .5, .6, .8),  ncol = 3))
#' plot_closest(matrix(c(.1, .2, .4, .5, .6, .8),  ncol = 3))
rgb2mnsl <- function(R, G = NULL, B = NULL){
    LUV.vals <- as(sRGB(R, G, B), "LUV")@coords
    # check for black
    if (any(LUV.vals[,"L"] == 0)){
      LUV.vals[LUV.vals[,"L"] == 0, ] <- 0
    }
    ncolors <- nrow(LUV.vals)
    dist.calc <- function(x, y) rowSums((rep(x, each = ncolors) - y) ^ 2)
    dists <- apply(munsell.map[, c("L", "U", "V")], 1, dist.calc, y = LUV.vals)
    if(is.null(dim(dists)))  closest <- which.min(dists)
    else closest <- apply(dists, 1, which.min)
    munsell.map[closest, "name"]
}

RGB2mnsl <- function(rgb.cols){
  LUV.vals <- as(rgb.cols, "LUV")@coords
  # check for black
  if (any(LUV.vals[,"L"] == 0)){
    LUV.vals[LUV.vals[,"L"] == 0, ] <- 0
  }
  ncolors <- nrow(LUV.vals)
  dist.calc <- function(x, y) rowSums((rep(x, each = ncolors) - y) ^ 2)
  dists <- apply(munsell.map[, c("L", "U", "V")], 1, dist.calc, y = LUV.vals)
  if(is.null(dim(dists)))  closest <- which.min(dists)
  else closest <- apply(dists, 1, which.min)
  munsell.map[closest, "name"]
}

#' Default munsell plot theme
#' 
#' Removes unnecessary clutter in plots
#' @keywords internal
#' @param bg.col takes colour to use as background colour
theme_munsell <- function(bg.col = "white") {
    ggplot2::theme(
      panel.grid.major = ggplot2::element_line(colour = NA),
      panel.grid.minor = ggplot2::element_line(colour = NA),
      panel.background = ggplot2::element_rect(fill = bg.col), 
      plot.background = ggplot2::element_blank(), 
      axis.line = ggplot2::element_line(colour = NA), 
      axis.ticks = ggplot2::element_blank(),
      axis.text = ggplot2::element_blank(),
      axis.title = ggplot2::element_blank(),
      legend.background = ggplot2::element_blank(),
      legend.key = ggplot2::element_blank(),
      legend.text = ggplot2::element_text(),
      legend.title = ggplot2::element_text())
}


#' Plot hex colours
#'
#' Quick way to look at a set of hex colours.
#' @param hex.colour character vector specifying colours in hex form
#' @param back.col specification of background colour of display
#' @return A ggplot object
#' @export
#' @examples
#' plot_hex("#000000")
#' plot_hex(c("#000000","#FFFFFF"))
plot_hex <- function(hex.colour,  back.col = "white"){
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 needed for this function to work. Please install it.",
      call. = FALSE)
  }
  if(length(hex.colour) == 1) {
    add.ops <- list(ggplot2::geom_text(ggplot2::aes(label = names)))
  }
  else add.ops <- list(ggplot2::facet_wrap(~ names))
  
  df <- data.frame(colour = hex.colour, 
                   names = factor(hex.colour, levels=hex.colour), 
                   x = 0, y = 0)
  ggplot2::ggplot(data = df,  ggplot2::aes(x = x,  y = y)) + 
    ggplot2::geom_tile(ggplot2::aes(fill = colour)) + 
    ggplot2::scale_fill_identity() + add.ops + 
    ggplot2::scale_x_continuous(expand = c(0, 0)) +
    ggplot2::scale_y_continuous(expand = c(0, 0)) +
    ggplot2::coord_fixed(ratio = 1) + theme_munsell(back.col)    
}

#' Plot a munsell colour
#'
#' Takes munsell text specifications and plots colour squares of them.
#' @param cols character vector specifying colours in Munsell form
#' @param back.col specification of background colour of display
#' @param ... passed to \code{\link{check_mnsl}}. Add fix = TRUE to fix "bad" colours()
#' @return A ggplot object
#' @export
#' @examples
#' plot_mnsl("5R 5/6")
#' plot_mnsl("5R 5/6",  back.col = "grey40")
#' p <- plot_mnsl(c("5R 6/6", "5Y 6/6", "5G 6/6", "5B 6/6", "5P 6/6"),
#'  back.col = "grey40")
#' p
#' # returned object is a ggplot object so we can alter the layout
#' summary(p)
#' p + ggplot2::facet_wrap(~ num, nrow = 1)
plot_mnsl <- function(cols,  back.col = "white", ...){
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 needed for this function to work. Please install it.",
      call. = FALSE)
  }
  add.ops <- NULL
  if(length(cols) > 1) {
     add.ops <- list(ggplot2::facet_wrap(~ num))
  }
  cols <- check_mnsl(cols)
  cols <- in_gamut(cols, ...)
  df <- data.frame(num = 1:length(cols), 
    names = factor(cols,  levels = c(unique(cols))),
    hex = mnsl2hex(cols), x = 0 , y = 0, stringsAsFactors = FALSE)
  df$labels <- factor(df$names,  levels = c(unique(cols), "NA"))
  df$labels[is.na(df$labels)] <- "NA"
  ggplot2::ggplot(data = df,  ggplot2::aes(x = x,  y = y)) + 
    ggplot2::geom_tile(ggplot2::aes(fill = hex)) + 
    add.ops +
    ggplot2::geom_text(ggplot2::aes(label = labels, 
      colour = text_colour(as.character(names)))) +
    ggplot2::scale_x_continuous(expand = c(0, 0))+
    ggplot2::scale_y_continuous(expand = c(0, 0))+
    ggplot2::coord_fixed() +  
    theme_munsell(back.col) +
    ggplot2::scale_fill_identity() + 
    ggplot2::scale_colour_identity() + 
    ggplot2::theme(strip.background = ggplot2::element_blank(),
      strip.text = ggplot2::element_blank())
}
    

#' Plot all colours with the same hue
#'
#' Plots slices of the Munsell colour system where hue is constant.
#' @param hue.name character vector of the desired hues. Or "all" for all hues.
#' @param back.col colour for the background
#' @return ggplot object
#' @export
#' @examples
#' hue_slice("5R")
#' hue_slice(c("5R", "5P"))
#' \dontrun{hue_slice("all")}
hue_slice <- function(hue.name = "all",  back.col = "white"){
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 needed for this function to work. Please install it.",
      call. = FALSE)
  }
  if (any(hue.name == "all")) {
    return(
      ggplot2::ggplot(ggplot2::aes(x = factor(chroma), y = factor(value)),
        data = munsell.map) +
      ggplot2::geom_tile(ggplot2::aes(fill = hex), colour = back.col) +
      ggplot2::facet_wrap(~ hue) +
      ggplot2::scale_x_discrete("Chroma", expand = c(0, 0)) + 
      ggplot2::coord_fixed(ratio = 1) +
      ggplot2::scale_y_discrete("Value", expand = c(0, 0)) +
      theme_munsell(back.col) +
      ggplot2::scale_fill_identity()
      )
  }
  else {
    if (!all(hue.name %in% munsell.map$hue)) stop("invalid hue names")
    ggplot2::ggplot(ggplot2::aes(x = factor(chroma), y = factor(value)), 
        data = subset(munsell.map, hue %in% hue.name)) +
      ggplot2::geom_tile(ggplot2::aes(fill = hex), colour = back.col, size = 1) +
      ggplot2::geom_text(ggplot2::aes(label = name, colour = text_colour(name)), 
        angle = 45, size = 2) +
      ggplot2::scale_colour_identity() +
      ggplot2::scale_x_discrete("Chroma") + 
      ggplot2::scale_y_discrete("Value", expand = c(0.125, 0)) +
      theme_munsell(back.col) +
      ggplot2::scale_fill_identity() +
      ggplot2::facet_wrap(~ hue)
  }
}

#' Plot all colours with the same value
#'
#' Plots slices of the Munsell colour system where value is constant.
#' @param value.name integer vector of the desired values. 
#' @param back.col colour for the background
#' @return ggplot object
#' @export
#' @examples
#' value_slice(2)
#' value_slice(c(2, 4))
#' # all values 
#' \dontrun{value_slice(1:10)}
value_slice <- function(value.name = 1:10,  back.col = "white"){
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 needed for this function to work. Please install it.",
      call. = FALSE)
  }
  if (!all(value.name %in% munsell.map$value)) stop("invalid Value")
  ggplot2::ggplot(ggplot2::aes(x = hue, y = factor(chroma)), 
      data = subset(munsell.map, value %in% value.name & hue != "N" & !is.na(hex))) +
    ggplot2::geom_tile(ggplot2::aes(fill = hex), colour = back.col) +
    ggplot2::coord_polar() +
    ggplot2::scale_x_discrete("Hue") + 
    ggplot2::scale_y_discrete("Chroma") +
    ggplot2::facet_wrap(~ value) +
    theme_munsell(back.col) +
    ggplot2::scale_fill_identity()
}

#' Plot all colours with the same chroma
#'
#' Plots slices of the Munsell colour system where chroma is constant.
#' @param chroma.name integer vector of the desired values. 
#' @param back.col colour for the background
#' @return ggplot object
#' @export
#' @examples
#' chroma_slice(2)
#' chroma_slice(18)
#' # Maybe want to delete text and add axis instead
#' p <- chroma_slice(18)
#' p$layers[[2]] <- NULL # remove text layer
#' p + ggplot2::theme(axis.text = ggplot2::element_text(), 
#'    axis.text.x = ggplot2::element_text(angle = 90, hjust = 1))
#' # all values 
#' \dontrun{chroma_slice(seq(0, 38, by = 2))}
chroma_slice <- function(chroma.name = seq(0, 26, by = 2),  back.col = "white"){
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 needed for this function to work. Please install it.",
      call. = FALSE)
  }
  if (!all(chroma.name %in% munsell.map$chroma)) stop("invalid Chroma")
  ggplot2::ggplot(ggplot2::aes(x = hue, y = value), 
      data = subset(munsell.map, chroma %in% chroma.name & hue != "N")) +
    ggplot2::geom_tile(ggplot2::aes(fill = hex), colour = back.col) +
    ggplot2::geom_text(ggplot2::aes(label = name, colour = text_colour(name)), 
      angle = 45, size = 2) +
    ggplot2::scale_colour_identity() +
    ggplot2::scale_x_discrete("Hue") + 
    ggplot2::scale_y_continuous("Value") +
    ggplot2::coord_fixed(ratio = 1/4) +
    ggplot2::facet_wrap(~ chroma) +
    theme_munsell(back.col) +
    ggplot2::scale_fill_identity() 
}

#' A vertical slice through the Munsell space
#'
#' Plot a hue and its complement at all values and chromas
#' @param hue.name character string of the desired hue. 
#' @param back.col colour for the background
#' @return ggplot object
#' @export
#' @examples
#' complement_slice("5PB")
#' complement_slice("5R")
#' complement_slice("10G")
complement_slice <- function(hue.name,  back.col = "white"){
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 needed for this function to work. Please install it.",
      call. = FALSE)
  }
  
  if (length(hue.name) > 1) stop("complement_slice currently only takes one hue")
  if (!hue.name %in% munsell.map$hue) stop("invalid hue name")
  comp.hue <- mnsl2hvc(complement(hvc2mnsl(hue.name, 2, 2)))$hue 
  munsell.sub <- subset(munsell.map, 
     hue == hue.name | hue == comp.hue)
  
  munsell.sub <- within(munsell.sub, {
    chroma <- ifelse(hue == comp.hue, -1, 1) * chroma
    hue <- factor(hue, levels = c(comp.hue, "N", hue.name))
    })
  
  ggplot2::ggplot(ggplot2::aes(x = chroma, y = value), 
      data = munsell.sub) + 
    ggplot2::geom_tile(ggplot2::aes(fill = hex), colour = back.col,  size = 1) +
    ggplot2::geom_text(ggplot2::aes(label = name, colour = text_colour(name)), 
      angle = 45, size = 2) +
    ggplot2::scale_fill_identity() +
    ggplot2::scale_colour_identity() +
    ggplot2::scale_x_continuous("Chroma") + 
    ggplot2::scale_y_continuous("Value") +
    ggplot2::facet_grid(. ~ hue,  scales = "free_x")  +
    theme_munsell(back.col)
  }

# 
# slice <- function(hue = NULL, chroma = NULL, value = NULL) {
#   if (!requireNamespace("ggplot2", quietly = TRUE)) {
#     stop("ggplot2 needed for this function to work. Please install it.",
#       call. = FALSE)
#   }
#   spec <- as.list(match.call())[-1]
#   cols <- merge(munsell:::munsell.map, expand.grid(spec))
#   vars <- names(spec)
#   varying <- c("hue", "chroma", "value")[!(c("hue", "chroma", "value") %in% vars)]
#   if (length(vars) == 1){
#     print(ggplot(cols, aes_string(varying[1], varying[2])) +
#       geom_tile(aes(fill = hex),  size = 1) +
#       geom_text(aes(label = name, colour = text_colour(name)), 
#         angle = 45, size = 2) +
#       scale_fill_identity() +
#       scale_colour_identity() +
#       coord_fixed() +
#       theme_munsell())
#   } else if (length(vars) == 2){
#     print(ggplot(cols, aes_string(varying[1], 1)) +
#       geom_tile(aes(fill = hex),  size = 1) +
#       geom_text(aes(label = name, colour = text_colour(name)), 
#         angle = 45, size = 2) +
#       scale_fill_identity() +
#       scale_colour_identity() +
#       coord_fixed() +
#       theme_munsell())
#   }
#   cols[order(cols$hue, cols$chroma, cols$value), "name"]
# }


#' Plot closest Munsell colour to an sRGB colour
#'
#' Take an sRGB colour and plots it along with the closest Munsell colour (using \code{\link{rgb2mnsl}} to find it)
#' @param R a numeric vector of red values or a 3 column matrix with the 
#' proportions R,  G,  B in the columns.
#' @param G numeric vector of green values
#' @param B numeric vector of blue values
#' @param back.col colour for the background
#' @seealso \code{\link{rgb2mnsl}}
#' @return ggplot object
#' @export
#' @examples
#' plot_closest(0.1, 0.1, 0.3)
#' plot_closest(matrix(c(.1, .2, .4, .5, .6, .8),  ncol = 3)) 
plot_closest <- function(R, G = NULL, B = NULL,  back.col = "white"){
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 needed for this function to work. Please install it.",
      call. = FALSE)
  }
  closest <- rgb2mnsl(R, G, B)
  ncolours <- length(closest)
  rgbnames <- apply(round(sRGB(R, G, B)@coords, 2), 1, paste, collapse = ", ")
  little.df <- data.frame(type = rep(c("actual", "closest"), each = ncolours),  
    hex = c(hex(sRGB(R,G,B)),  mnsl2hex(closest)), 
    name = c(rgbnames, closest), 
    x = rep(c(0, 0), each = ncolours), y = rep(1:ncolours, 2), 
    text.colour = rep(text_colour(closest), 2))
  ggplot2::ggplot(data = little.df, ggplot2::aes(x = x, y = y)) + 
    ggplot2::geom_tile(ggplot2::aes(fill = hex),
      colour = back.col, size = 2) +
    ggplot2::geom_text(ggplot2::aes(label = name, colour = text.colour), size = 2) +
    ggplot2::scale_colour_identity() +
    ggplot2::coord_fixed(ratio = 1) +
    theme_munsell(back.col) +
    ggplot2::scale_fill_identity()+
    ggplot2::facet_wrap(~ type)
}

#' Get text colour
#'
#' Get the appropriate text colour for writing on a munsell colour.
#' @param cols a character vector of munsell colours
#' @return a vector of "black" or "white"
#' @export
#' @keywords internal
text_colour <- function(cols){
  values <-  mnsl2hvc(cols)[, "value"]
  ifelse(values >4, "black", "white")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.r
\name{hvc2mnsl}
\alias{hvc2mnsl}
\title{Converts a hue, chroma and value to a Munsell colour}
\usage{
hvc2mnsl(hue, value = NULL, chroma = NULL, ...)
}
\arguments{
\item{hue}{a character vector of Munsell hues, or a 3 column data frame
containing the hue value and chroma levels}

\item{value}{a numeric vector of values}

\item{chroma}{a numeric vector of chromas}

\item{...}{passed on to \code{\link{check_mnsl}}. Use \code{fix = TRUE} to
fix "bad" colours}
}
\value{
a character string specification of a hex colour
}
\description{
Takes separate specifications of hue, value and chroma and returns the 
text specification of that colour.
}
\details{
Munsell colours are specified by hue, value and chroma.  They 
take a form like "5PB 5/10" where the first characters represent the
hue, followed by a space then the value and chroma separated by a "/". In
this package value should be an integer in 0:10 and chroma an even number
at most 24.  Note that not all possible specifications result in 
representable colours.  Regular recycling rules apply.
}
\examples{
hvc2mnsl("5PB", 5, 10)
# All values of 5PB with chroma 10
hvc2mnsl("5PB", 1:9, 10) # note some are undefined
plot_mnsl(hvc2mnsl("5PB", 1:9, 10))
}
\seealso{
\code{\link{check_mnsl}}, \code{\link{mnsl2hex}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alter.r
\name{lighter}
\alias{lighter}
\title{Make a munsell colour lighter}
\usage{
lighter(col, steps = 1)
}
\arguments{
\item{col}{character vector of Munsell colours}

\item{steps}{number of steps to take in increasing value}
}
\value{
character vector of Munsell colours
}
\description{
Increases the value of the Munsell colour.
}
\examples{
lighter("5PB 2/4")
cols <- c("5PB 2/4", "5Y 6/8")
p <- plot_mnsl(c(cols, lighter(cols), lighter(cols, 2)))
p + ggplot2::facet_wrap(~ names, ncol = 2)
# lighter and darker are usually reversible
lighter(darker("5PB 2/4"))
# unless you try to pass though white or black
lighter(darker("5PB 1/4"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alter.r
\name{saturate}
\alias{saturate}
\title{Make a munsell colour more saturated}
\usage{
saturate(col, steps = 1)
}
\arguments{
\item{col}{character vector of Munsell colours}

\item{steps}{number of steps to take in increasing chroma}
}
\value{
character vector of Munsell colours
}
\description{
Increases the chroma of the Munsell colour by step steps (multiples of 2).
}
\examples{
saturate("5PB 2/4")
cols <- c("5PB 2/2", "5Y 7/6")
p <- plot_mnsl(c(cols, saturate(cols), saturate(cols, 2)))
p + ggplot2::facet_wrap(~ names, ncol = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.r
\name{complement_slice}
\alias{complement_slice}
\title{A vertical slice through the Munsell space}
\usage{
complement_slice(hue.name, back.col = "white")
}
\arguments{
\item{hue.name}{character string of the desired hue.}

\item{back.col}{colour for the background}
}
\value{
ggplot object
}
\description{
Plot a hue and its complement at all values and chromas
}
\examples{
complement_slice("5PB")
complement_slice("5R")
complement_slice("10G")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alter.r
\name{rygbp}
\alias{rygbp}
\title{Change the hue of a munsell colour}
\usage{
rygbp(col, steps = 1)
}
\arguments{
\item{col}{character vector of Munsell colours}

\item{steps}{number of hue steps to take}
}
\value{
character vector of Munsell colours
}
\description{
Moves the hue of a munsell colour in the direction red->yellow->green->blue->purple->red
}
\examples{
my_red <- "10R 4/8"
rygbp(my_red)
plot_mnsl(c(my_red, rygbp(my_red, 2), rygbp(my_red, 4)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.r
\name{in_gamut}
\alias{in_gamut}
\title{Checks if a Munsell colour is defined in RGB space}
\usage{
in_gamut(col, fix = FALSE)
}
\arguments{
\item{col}{a character vector representing Munsell colours.}

\item{fix}{passed on to \code{\link{fix_mnsl}}. Use \code{fix = TRUE} to
fix "bad" colours}
}
\value{
a character vector containing the input colours.  If any colours
were outside the gamut they will be represented by NA.
}
\description{
Not all possible correctly formatted Munsell colours result in a colour
representable in RGB space.  This function checks if the colour is
representable.
}
\examples{
in_gamut(c("5R 5/8","2.5R 9/28"))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.r
\name{plot_hex}
\alias{plot_hex}
\title{Plot hex colours}
\usage{
plot_hex(hex.colour, back.col = "white")
}
\arguments{
\item{hex.colour}{character vector specifying colours in hex form}

\item{back.col}{specification of background colour of display}
}
\value{
A ggplot object
}
\description{
Quick way to look at a set of hex colours.
}
\examples{
plot_hex("#000000")
plot_hex(c("#000000","#FFFFFF"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.r
\name{value_slice}
\alias{value_slice}
\title{Plot all colours with the same value}
\usage{
value_slice(value.name = 1:10, back.col = "white")
}
\arguments{
\item{value.name}{integer vector of the desired values.}

\item{back.col}{colour for the background}
}
\value{
ggplot object
}
\description{
Plots slices of the Munsell colour system where value is constant.
}
\examples{
value_slice(2)
value_slice(c(2, 4))
# all values 
\dontrun{value_slice(1:10)}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alter.r
\name{seq_mnsl}
\alias{seq_mnsl}
\title{Generate a sequence of Munsell colours}
\usage{
seq_mnsl(from, to, n, fix = FALSE)
}
\arguments{
\item{from}{character string of first Munsell colour}

\item{to}{character string of last Munsell colour}

\item{n}{number of colours in sequence}

\item{fix}{Should colours outside of the gamut be fixed? 
Passed on to \code{\link{fix_mnsl}}}
}
\value{
character vector of Munsell colours
}
\description{
Generates a sequence of Munsell colours.  The sequence is generated by 
finding the closest munsell colours to a equidistant sequence of colours in  #' LUV space.
}
\examples{
seq_mnsl("5R 2/4", "5R 5/16", 4)
plot_mnsl(seq_mnsl("5R 2/4", "5R 5/16", 4))
plot_mnsl(seq_mnsl("5R 5/6", 
  complement("5R 5/6"), 5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.r
\name{text_colour}
\alias{text_colour}
\title{Get text colour}
\usage{
text_colour(cols)
}
\arguments{
\item{cols}{a character vector of munsell colours}
}
\value{
a vector of "black" or "white"
}
\description{
Get the appropriate text colour for writing on a munsell colour.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alter.r
\name{darker}
\alias{darker}
\title{Make a munsell colour darker}
\usage{
darker(col, steps = 1)
}
\arguments{
\item{col}{character vector of Munsell colours}

\item{steps}{number of steps to take in decreasing value}
}
\value{
character vector of Munsell colours
}
\description{
Decreases the value of the Munsell colour by 1.
}
\examples{
darker("5PB 3/4")
cols <- c("5PB 3/4", "5Y 7/8")
p <- plot_mnsl(c(cols, darker(cols), darker(cols, 2)))
p + ggplot2::facet_wrap(~ names, ncol = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alter.r
\name{pbgyr}
\alias{pbgyr}
\title{Change the hue of a munsell colour}
\usage{
pbgyr(col, steps = 1)
}
\arguments{
\item{col}{character vector of Munsell colours}

\item{steps}{number of hue steps to take}
}
\value{
character vector of Munsell colours
}
\description{
Moves the hue of a munsell colour in the direction purple->blue->green->yellow->red->purple
}
\examples{
my_red <- "2.5R 4/8"
pbgyr(my_red)
plot_mnsl(c(my_red, pbgyr(my_red, 2), pbgyr(my_red, 4)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alter.r
\name{complement}
\alias{complement}
\title{Find the complement of a munsell colour}
\usage{
complement(col, ...)
}
\arguments{
\item{col}{character vector of Munsell colours}

\item{...}{deprecated}
}
\value{
character vector of Munsell colours
}
\description{
Finds the munsell colour with the same chroma and value but on the opposite
side of the hue circle. The complement is not defined 
for greys (hue == "N"), and the function returns the grey untransformed.
}
\examples{
complement("5PB 2/4")
cols <- c("5PB 2/4", "5Y 7/8")
plot_mnsl(c(cols, complement(cols)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alter.r
\name{desaturate}
\alias{desaturate}
\title{Make a munsell colour less saturated}
\usage{
desaturate(col, steps = 1)
}
\arguments{
\item{col}{character vector of Munsell colours}

\item{steps}{number of steps to take in decreasing chroma}
}
\value{
character vector of Munsell colours
}
\description{
Decreases the chroma of the Munsell colour by one step steps (multiples of 2).
}
\examples{
desaturate("5PB 2/4")
cols <- c("5PB 2/6", "5Y 7/8")
p <- plot_mnsl(c(cols, desaturate(cols), desaturate(cols, 2)))
p + ggplot2::facet_wrap(~ names, ncol = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.r
\name{plot_mnsl}
\alias{plot_mnsl}
\title{Plot a munsell colour}
\usage{
plot_mnsl(cols, back.col = "white", ...)
}
\arguments{
\item{cols}{character vector specifying colours in Munsell form}

\item{back.col}{specification of background colour of display}

\item{...}{passed to \code{\link{check_mnsl}}. Add fix = TRUE to fix "bad" colours()}
}
\value{
A ggplot object
}
\description{
Takes munsell text specifications and plots colour squares of them.
}
\examples{
plot_mnsl("5R 5/6")
plot_mnsl("5R 5/6",  back.col = "grey40")
p <- plot_mnsl(c("5R 6/6", "5Y 6/6", "5G 6/6", "5B 6/6", "5P 6/6"),
 back.col = "grey40")
p
# returned object is a ggplot object so we can alter the layout
summary(p)
p + ggplot2::facet_wrap(~ num, nrow = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.r
\name{mnsl_hues}
\alias{mnsl_hues}
\title{Munsell hues}
\usage{
mnsl_hues()
}
\value{
a character vector containing the hue values.
}
\description{
Returns a character vector of the Munsell hues in hue order starting at 2.5R and excluding grey ("N").
}
\examples{
mnsl_hues()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.r
\name{hue_slice}
\alias{hue_slice}
\title{Plot all colours with the same hue}
\usage{
hue_slice(hue.name = "all", back.col = "white")
}
\arguments{
\item{hue.name}{character vector of the desired hues. Or "all" for all hues.}

\item{back.col}{colour for the background}
}
\value{
ggplot object
}
\description{
Plots slices of the Munsell colour system where hue is constant.
}
\examples{
hue_slice("5R")
hue_slice(c("5R", "5P"))
\dontrun{hue_slice("all")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.r
\name{theme_munsell}
\alias{theme_munsell}
\title{Default munsell plot theme}
\usage{
theme_munsell(bg.col = "white")
}
\arguments{
\item{bg.col}{takes colour to use as background colour}
}
\description{
Removes unnecessary clutter in plots
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.r
\name{fix_mnsl}
\alias{fix_mnsl}
\title{Fix an undefined Munsell colour}
\usage{
fix_mnsl(col)
}
\arguments{
\item{col}{a character vector representing Munsell colours.}
}
\value{
a character vector containing the fixed colours.
}
\description{
Takes correctly specified but undefined colours and outputs something
sensible.  Normally this happens when the chroma is too high.  So,  here
sensible means the colour with the same hue and value and maximum defined
chroma.
}
\examples{
fix_mnsl(c("5R 5/8","2.5R 9/28"))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.r
\name{rgb2mnsl}
\alias{rgb2mnsl}
\title{Converts an sRGB colour to Munsell}
\usage{
rgb2mnsl(R, G = NULL, B = NULL)
}
\arguments{
\item{R}{a numeric vector of red values or a 3 column matrix with the
proportions R,  G,  B in the columns.}

\item{G}{numeric vector of green values}

\item{B}{numeric vector of blue values}
}
\description{
Finds the closest Munsell colour (in LUV space) to the specified sRGB colour
}
\examples{
rgb2mnsl(0.1, 0.1, 0.3)
rgb2mnsl(matrix(c(.1, .2, .4, .5, .6, .8),  ncol = 3))
plot_closest(matrix(c(.1, .2, .4, .5, .6, .8),  ncol = 3))
}
\seealso{
\code{\link{plot_closest}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.r
\name{chroma_slice}
\alias{chroma_slice}
\title{Plot all colours with the same chroma}
\usage{
chroma_slice(chroma.name = seq(0, 26, by = 2), back.col = "white")
}
\arguments{
\item{chroma.name}{integer vector of the desired values.}

\item{back.col}{colour for the background}
}
\value{
ggplot object
}
\description{
Plots slices of the Munsell colour system where chroma is constant.
}
\examples{
chroma_slice(2)
chroma_slice(18)
# Maybe want to delete text and add axis instead
p <- chroma_slice(18)
p$layers[[2]] <- NULL # remove text layer
p + ggplot2::theme(axis.text = ggplot2::element_text(), 
   axis.text.x = ggplot2::element_text(angle = 90, hjust = 1))
# all values 
\dontrun{chroma_slice(seq(0, 38, by = 2))}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.r
\name{mnsl}
\alias{mnsl}
\alias{mnsl2hex}
\title{Converts a Munsell colour to hex}
\usage{
mnsl(col, ...)
}
\arguments{
\item{col}{a character string representing a Munsell colour.}

\item{...}{passed on to \code{\link{in_gamut}}. Use \code{fix = TRUE} to
fix "bad" colours}
}
\value{
a character string specification of a hex colour
}
\description{
Take a character string representation of a Munsell colour and returns the 
hex specification of that colour
}
\details{
Munsell colours are specified by hue, value and chroma.  They 
take a form like "5PB 5/10" where the first characters represent the
hue, followed by a space then the value and chroma separated by a "/". In
this package value should be an integer in 0:10 and chroma an even number
at most 24.  Note that not all possible specifications result in 
representable colours.
}
\examples{
mnsl2hex("5PB 5/10")
# use a munsell colour in a plot
plot.new()
rect(0, 0, 1 ,1 , col = mnsl("5R 5/10"))
}
\seealso{
\code{\link{check_mnsl}},\code{\link{in_gamut}}, \code{\link{hvc2mnsl}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/munsell-package.R
\docType{package}
\name{munsell}
\alias{munsell-package}
\alias{munsell}
\alias{package-munsell}
\title{Munsell colour system.}
\description{
This package makes it easy to access and manipulate the colours in the 
munsell colour system. The conversion from munsell specifications to sRGB based on the renotation data from  \url{http://www.cis.rit.edu/mcsl/online/munsell.php} which is a digitization of Table 1 in Newhall, Nickerson & Judd (1943).  The code for conversion can be found in the package directory in inst/raw/getmunsellmap.r
}
\references{
S. M. Newhall, D. Nickerson, and D. B. Judd. Final report of the O.S.A. subcommittee on the spacing of the munsell colors. J. Opt. Soc. Am., 33(7):385-411, 07 1943.

Munsell Renotation Data, RIT Munsell Color Science Laboratory.  \url{http://www.cis.rit.edu/mcsl/online/munsell.php}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://cran.r-project.org/package=munsell}
  \item \url{https://github.com/cwickham/munsell/}
  \item Report bugs at \url{https://github.com/cwickham/munsell/issues}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.r
\name{plot_closest}
\alias{plot_closest}
\title{Plot closest Munsell colour to an sRGB colour}
\usage{
plot_closest(R, G = NULL, B = NULL, back.col = "white")
}
\arguments{
\item{R}{a numeric vector of red values or a 3 column matrix with the 
proportions R,  G,  B in the columns.}

\item{G}{numeric vector of green values}

\item{B}{numeric vector of blue values}

\item{back.col}{colour for the background}
}
\value{
ggplot object
}
\description{
Take an sRGB colour and plots it along with the closest Munsell colour (using \code{\link{rgb2mnsl}} to find it)
}
\examples{
plot_closest(0.1, 0.1, 0.3)
plot_closest(matrix(c(.1, .2, .4, .5, .6, .8),  ncol = 3)) 
}
\seealso{
\code{\link{rgb2mnsl}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.r
\name{mnsl2hvc}
\alias{mnsl2hvc}
\title{Converts a Munsell colour to a hue, chroma and value triplet}
\usage{
mnsl2hvc(col, ...)
}
\arguments{
\item{col}{a character vector of Munsell colours}

\item{...}{passed on to \code{\link{check_mnsl}}. Use \code{fix = TRUE} to
fix "bad" colours}
}
\value{
a data frame with named columns hue, value and chroma containing the hue,
value and chroma levels.
}
\description{
Takes a text specification of a Munsell colour and returns
the hue, chroma and value triplet.
}
\details{
Munsell colours are specified by hue, value and chroma.  They 
take a form like "5PB 5/10" where the first characters represent the
hue, followed by a space then the value and chroma separated by a "/". In
this package value should be an integer in 0:10 and chroma an even number
at most 24.  Note that not all possible specifications result in 
representable colours.
}
\examples{
mnsl2hvc("5PB 5/10")
hvc2mnsl(mnsl2hvc("5PB 5/10"))
}
\seealso{
\code{\link{check_mnsl}}, \code{\link{hvc2mnsl}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.r
\name{check_mnsl}
\alias{check_mnsl}
\title{Checks for valid Munsell colours}
\usage{
check_mnsl(col)
}
\arguments{
\item{col}{a character vector representing Munsell colours.}
}
\value{
a character vector containing the input colours.  If any colours
were outside the gamut they will be represented by NA.
}
\description{
Checks user supplied munsell specification for validity.  
I.e. colour is of form "h v/c" and h,  v and c take valid values.
}
\examples{
check_mnsl(c("5R 5/8","2.5R 9/28"))
}
\keyword{internal}
